@@ -583,7 +583,9 @@ get_mount_point_for_mount (GUnixMountEntry *mount_entry)
     {
       GUnixMountPoint *mount_point = l->data;
       if (g_strcmp0 (g_unix_mount_get_mount_path (mount_entry),
-                     g_unix_mount_point_get_mount_path (mount_point)) == 0)
+                     g_unix_mount_point_get_mount_path (mount_point)) == 0 &&
+          g_strcmp0 (g_unix_mount_get_device_path (mount_entry),
+                     g_unix_mount_point_get_device_path (mount_point)) == 0)
         {
           ret = mount_point;
           goto out;
@@ -700,6 +702,9 @@ should_include_mount (GVfsUDisks2VolumeMonitor  *monitor,
   GUnixMountPoint *mount_point;
   gboolean ret;
 
+  if (g_unix_mount_is_system_internal (mount_entry))
+    return FALSE;
+
   /* if mounted at the designated mount point, use that info to decide */
   mount_point = get_mount_point_for_mount (mount_entry);
   if (mount_point != NULL)
@@ -747,12 +752,14 @@ should_include_volume_check_mount_points (GVfsUDisks2VolumeMonitor *monitor,
       mount_entry = g_unix_mount_at (mount_point, NULL);
       if (mount_entry != NULL)
         {
-          if (!should_include_mount (monitor, mount_entry))
+          if (should_include_mount (monitor, mount_entry))
             {
               g_unix_mount_free (mount_entry);
-              ret = FALSE;
+              ret = TRUE;
               goto out;
             }
+
+          ret = FALSE;
           g_unix_mount_free (mount_entry);
         }
     }
